/*
 * Copyright (c) 2009 The openGion Project.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.opengion.plugin.column;

import org.opengion.hayabusa.common.HybsSystem;
import org.opengion.hayabusa.db.AbstractRenderer;
import org.opengion.hayabusa.db.CellRenderer;
import org.opengion.hayabusa.db.DBColumn;

/**
 * SLABEL レンデラーは、桁数の長いデータをコンパクトに表示させる
 * LABEL レンデラーの類似クラスです。
 *
 * 全角２Byte / 半角および半角カタカナを １Byte で簡易計算し、指定の
 * 桁数でカットします。
 * 初期値は、20Byteで、桁数は、表示パラメータ（RENDERER_PARAM)で指定します。
 * 文字をカットした場合は、最後に『...』を追加し、カット前の文字を title 属性に
 * 設定することで、マウスをカット後の文字に載せると、カット前の値がチップ表示
 * されます。
 * &lt;span title="カット前の値"&gt;カット文字...&lt;/span&gt;
 * カットされなかった場合は、元の文字がそのまま表示されます。
 *
 *  カラムの表示に必要な属性は, DBColumn オブジェクト より取り出します。
 * このクラスは、表示パラメータになにも指定しない（デフォルト）場合は、
 * すべて同一のオブジェクトを返します。それ以外は、DBColumn オブジェクト毎に１つ作成されます。
 *
 * @og.rev 3.5.6.2 (2004/07/05) 新規作成
 * @og.group データ表示
 *
 * @version  4.0
 * @author	 Kazuhiko Hasegawa
 * @since    JDK5.0,
 */
public class Renderer_SLABEL extends AbstractRenderer {
	//* このプログラムのVERSION文字列を設定します。	{@value} */
	private static final String VERSION = "4.0.0 (2005/08/31)" ;

	private static final CellRenderer dbCell = new Renderer_SLABEL() ;	// 20Byteでカット
	private final int cutSize;

	/**
	 * デフォルトコンストラクター。
	 * このコンストラクターで、基本オブジェクトを作成します。
	 *
	 */
	public Renderer_SLABEL() {
		cutSize = 20;
	}

	/**
	 * デフォルトコンストラクター。
	 *
	 * @param   clm DBColumn オブジェクト
	 */
	private Renderer_SLABEL( final DBColumn clm ) {
		String param = clm.getRendererParam();
		cutSize = Integer.parseInt( param );
	}

	/**
	 * 各オブジェクトから自分のインスタンスを返します。
	 * 自分自身をキャッシュするのか、新たに作成するのかは、各サブクラスの実装に
	 * まかされます。
	 *
	 * @param   clm DBColumn オブジェクト
	 * @return  CellRenderer オブジェクト
	 */
	public CellRenderer newInstance( final DBColumn clm ) {
		String param = clm.getRendererParam();

		if( param != null && param.length() > 0 ) {
			return new Renderer_SLABEL( clm );
		}
		else {
			return dbCell;
		}
	}

	/**
	 * データの表示用文字列を返します。
	 *
	 * 全角２Byte / 半角および半角カタカナを １Byte で簡易計算し、指定の
	 * 桁数でカットします。
	 * 初期値は、20Byteで、桁数は、表示パラメータ（RENDERER_PARAM)で指定します。
	 *
	 * @param	value String
	 * @return	データの表示用文字列
	 */
	public String getValue( final String value ) {

		// 簡易的処理。すべてが全角であっても、制限以内である。
		int len      = value.length();
		if( len*2 <= cutSize ) { return value; }

		int byteSize = 0;
		int adrs;
		for( adrs=0; adrs<len && byteSize<cutSize ; adrs++ ) {
			char ch = value.charAt(adrs);
			if( ch <= 0x7f || ( ch >= 0xff61 && ch <= 0xff9f ) ) {
				byteSize ++;
			}
			else {
				byteSize +=2;
			}
		}

		// 正確にカウントした結果、制限以内であったため。
		if( adrs==len && byteSize<=cutSize ) {
			return value;
		}
		else if( byteSize>cutSize ) {		// オーバーした場合
			adrs-- ;
		}

		StringBuilder buf = new StringBuilder( HybsSystem.BUFFER_SMALL );
		buf.append( "<span title=\"" );
		buf.append( value );
		buf.append( "\">" );
		buf.append( value.substring( 0,adrs ) );	// 切り出し
		buf.append( "...</span>" );

		return buf.toString();
	}
}
